@@ -1,6 +1,6 @@
 // -*- C++ -*-
 
-// Copyright (C) 2005-2018 Free Software Foundation, Inc.
+// Copyright (C) 2005-2021 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the terms
@@ -61,10 +61,10 @@ namespace __gnu_pbds
       struct entry_pred<_VTp, Pred, _Alloc, false>
       {
       private:
-	typedef typename _Alloc::template rebind<_VTp>		__rebind_v;
+	typedef rebind_traits<_Alloc, _VTp>		__rebind_v;
 
       public:
-	typedef typename __rebind_v::other::const_pointer	entry;
+	typedef typename __rebind_v::const_pointer	entry;
 
 	struct type : public Pred
 	{
